<?php

/**
 * Request helper for sare wp synchronization
 *
 * @link       https://sare.pl/
 * @since      1.0.0
 *
 * @package    Sare_Wp
 * @subpackage Sare_Wp/includes
 */

/**
 * Request helper for sare wp synchronization
 *
 *
 * @package    Sare_Wp
 * @subpackage Sare_Wp/includes
 * @author     Sare <hello@sare.pl>
 */
class Sare_Wp_Request_Helper
{

    const HASH_ALGORITHM = 'SHA256';
    const VERSION = 'wc/v3';

    private ?string $consumerSecret = null;
    private array $requests = [];

    /**
     * Initialize the collections used to maintain the actions and filters.
     *
     * @since    1.0.0
     */
    public function __construct()
    {
    }

    public function setConsumerSecret($consumerSecret): void
    {
        $this->consumerSecret = $consumerSecret;
    }

    protected function getSecret()
    {
        return $this->consumerSecret;
    }

    public function generateAuthSignature($method, $requestUrl, $queryStringParams): string
    {
        //eq. GET&https://www.example.com?param1&param2
        $stringToSign = $method . '&' . $requestUrl . '?' . http_build_query($queryStringParams);
        $secret = $this->getSecret();
        return \base64_encode(\hash_hmac(self::HASH_ALGORITHM, $stringToSign, $secret));
    }

    public function getConfigObject(): Sare_Wp_Config_Object
    {
        require_once plugin_dir_path(__FILE__) . 'class-sare-wp-config-object.php';
        $shopUrl = get_site_url();
        $hash = $this->generateAuthSignature('GET', SARE_WP_URL . '/config-woocommerce', ['shopUrl' => $shopUrl]);
        $response = wp_remote_get(SARE_WP_URL . '/config-woocommerce?hash=' . $hash . '&shopUrl=' . $shopUrl);
        $responseCode = wp_remote_retrieve_response_code($response);
        if (is_array($response) && !is_wp_error($response) && $responseCode === 200) {
            $body = json_decode(wp_remote_retrieve_body($response), true);
            $configObject = new Sare_Wp_Config_Object($body);
        } else {
            $configObject = new Sare_Wp_Config_Object([]);
        }

        return $configObject;
    }

    public function postConfigObject($data): int
    {
        $hash = $this->generateAuthSignature(
            'POST',
            SARE_WP_URL . '/config-woocommerce',
            ['shopUrl' => get_site_url()]
        );
        $response = wp_remote_post(
            SARE_WP_URL . '/config-woocommerce?hash=' . $hash . '&shopUrl=' . get_site_url(),
            [
                'method' => 'POST',
                'timeout' => 30,
                'sslverify' => true,
                'headers' => ['Content-Type' => 'application/json; charset=utf-8'],
                'body' => json_encode($data),
                'data_format' => 'body'
            ]
        );
        $responseCode = wp_remote_retrieve_response_code($response);
        return (int) $responseCode;
    }

    public function sendWebhook(int $customerId = null, string $type): bool
    {
        $data = ['customer_id' => $customerId, 'type' => $type];
        if (in_array(md5($data['customer_id']), $this->requests, true)) {
            return false;
        }
        if ($type === 'client.delete') {
            $user_info = get_userdata($customerId);
            $data['email'] = $user_info->user_email;
        }
        $hash = $this->generateAuthSignature(
            'POST',
            SARE_WP_URL . '/wh-woocommerce',
            ['shopUrl' => get_site_url()]
        );
        $response = wp_remote_post(
            SARE_WP_URL . '/wh-woocommerce?hash=' . $hash . '&shopUrl=' . get_site_url(),
            [
                'method' => 'POST',
                'timeout' => 15,
                'sslverify' => true,
                'headers' => ['Content-Type' => 'application/json; charset=utf-8'],
                'body' => json_encode($data),
                'data_format' => 'body'
            ]
        );
        $responseCode = wp_remote_retrieve_response_code($response);
        $this->requests[] = md5($data['customer_id']);
        if (is_array($response) && !is_wp_error($response) && $responseCode === 200) {
            $log = sprintf('Successfully send webhook: %s to sare, with customer id: %d', $type, $customerId);
            $result = true;
        } else {
            $log = sprintf('Fail to send webhook: %s to sare, with customer id: %d', $type, $customerId);
            $result = false;
        }
        $logger = wc_get_logger();
        $context = array('source' => 'sarewp');
        $logger->info($log, $context);

        return $result;
    }

}
