<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://sare.pl/
 * @since      1.0.0
 *
 * @package    Sare_Wp
 * @subpackage Sare_Wp/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Sare_Wp
 * @subpackage Sare_Wp/includes
 * @author     Sare <hello@sare.pl>
 */
class Sare_Wp
{

    /**
     * The loader that's responsible for maintaining and registering all hooks that power
     * the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      Sare_Wp_Loader $loader Maintains and registers all hooks for the plugin.
     */
    protected $loader;

    /**
     * The unique identifier of this plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string $pluginName The string used to uniquely identify this plugin.
     */
    protected $pluginName;

    /**
     * The current version of the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string $version The current version of the plugin.
     */
    protected $version;

    protected $requestHelper;

    /**
     * Define the core functionality of the plugin.
     *
     * Set the plugin name and the plugin version that can be used throughout the plugin.
     * Load the dependencies, define the locale, and set the hooks for the admin area and
     * the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function __construct()
    {
        if (defined('SARE_WP_VERSION')) {
            $this->version = SARE_WP_VERSION;
        } else {
            $this->version = '1.0.0';
        }
        $this->pluginName = 'sare-wp';

        $this->loadDependencies();
        $this->setRequestHelper();
        $this->setLocale();
        $this->defineAdminHooks();
        $this->definePublicHooks();
    }

    /**
     * Load the required dependencies for this plugin.
     *
     * Include the following files that make up the plugin:
     *
     * - Sare_Wp_Loader. Orchestrates the hooks of the plugin.
     * - Sare_Wp_i18n. Defines internationalization functionality.
     * - Sare_Wp_Admin. Defines all hooks for the admin area.
     * - Sare_Wp_Public. Defines all hooks for the public side of the site.
     *
     * Create an instance of the loader which will be used to register the hooks
     * with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private function loadDependencies()
    {
        /**
         * The class responsible for orchestrating the actions and filters of the
         * core plugin.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-sare-wp-loader.php';

        /**
         * The class responsible for defining internationalization functionality
         * of the plugin.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-sare-wp-i18n.php';

        /**
         * The class responsible for defining all actions that occur in the admin area.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'admin/class-sare-wp-admin.php';

        /**
         * The class responsible for defining all actions that occur in the public-facing
         * side of the site.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'public/class-sare-wp-public.php';

        /**
         * The class responsible for connect with Sare remote API.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-sare-wp-request-helper.php';

        $this->loader = new Sare_Wp_Loader();
    }

    /**
     * Define the locale for this plugin for internationalization.
     *
     * Uses the Sare_Wp_i18n class in order to set the domain and to register the hook
     * with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private function setLocale()
    {
        $plugin_i18n = new Sare_Wp_i18n();

        $this->loader->addAction('plugins_loaded', $plugin_i18n, 'load_plugin_textdomain');
    }

    /**
     * Register all of the hooks related to the admin area functionality
     * of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function defineAdminHooks()
    {
        $plugin_admin = new Sare_Wp_Admin();

        $this->loader->addAction('admin_init', $plugin_admin, 'checkThatWoocommerceIsActive');
        $this->loader->addAction('init', $plugin_admin, 'sareStartSession');
        $this->loader->addAction('admin_menu', $plugin_admin, 'sareAdminMenu');
        $this->loader->addAction('admin_notices', $plugin_admin, 'sareConnectorResult');
        $this->loader->addAction('admin_notices', $plugin_admin, 'sarePluginResult');
        $this->loader->addAction('admin_post_sare_config', $plugin_admin, 'handleSareConfig');
        $this->loader->addAction('admin_post_sare_sync_all', $plugin_admin, 'handleSareSyncAll');
    }

    /**
     * Register all of the hooks related to the public-facing functionality
     * of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function definePublicHooks()
    {
        $plugin_public = new Sare_Wp_Public($this->requestHelper);

        if ($this->requestHelper !== null) {
            $this->loader->addAction('user_register', $plugin_public, 'saveNewUser');
            $this->loader->addAction('delete_user', $plugin_public, 'deleteUser');
            $this->loader->addAction('profile_update', $plugin_public, 'updateCustomer');
            $this->loader->addAction('woocommerce_new_customer', $plugin_public, 'saveNewCustomer');
            $this->loader->addAction('woocommerce_update_customer', $plugin_public, 'updateCustomer');
            $this->loader->addAction('woocommerce_delete_customer', $plugin_public, 'deleteCustomer');
        }
    }

    /**
     * Run the loader to execute all of the hooks with WordPress.
     *
     * @since    1.0.0
     */
    public function run()
    {
        $this->loader->run();
    }

    /**
     * The name of the plugin used to uniquely identify it within the context of
     * WordPress and to define internationalization functionality.
     *
     * @return    string    The name of the plugin.
     * @since     1.0.0
     */
    public function getPluginName()
    {
        return $this->pluginName;
    }

    /**
     * The reference to the class that orchestrates the hooks with the plugin.
     *
     * @return    Sare_Wp_Loader    Orchestrates the hooks of the plugin.
     * @since     1.0.0
     */
    public function getLoader()
    {
        return $this->loader;
    }

    /**
     * Retrieve the version number of the plugin.
     *
     * @return    string    The version number of the plugin.
     * @since     1.0.0
     */
    public function getVersion()
    {
        return $this->version;
    }

    /**
     * @return Sare_Wp_Request_Helper|null
     */
    public function setRequestHelper()
    {
        global $wpdb;
        $result = $wpdb->get_row(
            "SELECT * FROM {$wpdb->prefix}woocommerce_api_keys WHERE description LIKE 'SARE WP%'",
            ARRAY_A
        );
        if ($result !== null) {
            $this->requestHelper = new Sare_Wp_Request_Helper();
            $this->requestHelper->setConsumerSecret($result['consumer_secret']);
        } else {
            $this->requestHelper = null;
        }
    }

}
